/*
 * Copyright (c) 1998-2006 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * The contents of this file constitute Original Code as defined in and
 * are subject to the Apple Public Source License Version 2.0 (the
 * "License").  You may not use this file except in compliance with the
 * License.  Please obtain a copy of the License at
 * http://www.apple.com/publicsource and read it before using this file.
 * 
 * This Original Code and all software distributed under the License are
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@
 */

#ifndef _APPLE_SMBIOS_H
#define _APPLE_SMBIOS_H

#include <IOKit/IOService.h>
#include <IOKit/IOKitKeys.h>
#include "SMBIOS.h"

class SMBPackedStrings;

class AppleSMBIOSEFI : public IOService
{
    OSDeclareDefaultStructors( AppleSMBIOSEFI )

protected:
    IOMemoryMap *           fDMIMemoryMap;

    OSData *                memSlotsData;
    OSData *                memTypesData;
    OSData *                memSizesData;
    OSData *                memSpeedData;
    OSData *                memInfoData;
    OSData *                memManufData;
    OSData *                memSerialData;
    OSData *                memPartData;
    UInt64                  memSizeTotal;
	
	void *					SMBIOSTable;
	UInt16					SMBIOSTableLength;

    enum { kMemDataSize = 64 };

    enum {
        kNoMemoryInfo,
        kMemoryModuleInfo,
        kMemoryDeviceInfo
    }                       memInfoSource;

    bool                    findSMBIOSTableEFI( void );

    void                    decodeSMBIOSTable(
                                const void *            tableData,
                                UInt16                  tableLength,
                                UInt16                  structureCount );

    void                    decodeSMBIOSStructure(
                                const SMBStructHeader * structureHeader,
                                const void *            tableBoundary );

    void                    processSMBIOSStructure(
                                const SMBBIOSInformation *      bios,
                                SMBPackedStrings *              strings );

    void                    processSMBIOSStructure(
                                const SMBSystemInformation *    sys,
                                SMBPackedStrings *              strings );

    void                    processSMBIOSStructure(
                                const SMBProcessorInformation * cpu,
                                SMBPackedStrings *              strings );

    void                    processSMBIOSStructure(
                                const SMBMemoryDevice *         memory,
                                SMBPackedStrings *              strings );

    void                    processSMBIOSStructure(
                                const SMBMemoryModule *         memory,
                                SMBPackedStrings *              strings );

    void                    updateDeviceTree( void );

public:
    virtual bool            start( IOService * provider );

    virtual void            free( void );
};

#endif /* !_APPLE_SMBIOS_H */
